/*
 * ConstMemory Linked List header
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.6 $  $Date: 2003/01/04 12:43:19 $
 * ---------------------------------------------------------------------------
 */
#ifndef _SHM_LINKED_LIST
# define _SHM_LINKED_LIST

#include <phlib/xl_types.h>
#include <phlib/xl_linklst.h>

typedef struct {
  int Bnext;    /* BLOCK NUMBER of the previous LIST NODE */
  int Bprev;    /* BLOCK NUMBER of the next LIST NODE     */
  int nodesize; 
  int datalen;  /* Length of data held in node */
  int o_data;   /* Offset to data start */
} SNode;

typedef struct {
  int     o_qmap;     /* Offset to where map starts           */
  int     o_qroot;    /* Offset to where our node pool starts */
  int     bo_head;    /* BLOCK NUMBER of the list HEAD        */
  int     bo_last;    /* BLOCK NUMBER of the LAST BLOCK       */
  int     bo_current; /* BLOCK NUMBER of the CURRENT BLOCK    */
  int     datasize;   /* Size of user data stored in list     */
  int     nodesize;   /* size of whole node inc user data     */
  int     maxnodes;   /* Maximum number of nodes allowed      */
  int     count;      /* Number of nodes in list              */
  int     mapsize;    /* Mapsize in BYTES                     */
  int     blocksize;  /* Size of an allocation unit           */
} SHMLIST;

#define shm_List_insert_node_len(a,b,c,d) shm_List_insert_a_node((a),(b),(c),1,d)
#define shm_List_insert_node(a,b,c) shm_List_insert_a_node((a),(b),(c),1,a->datasize)
#define shm_List_insert_space(a,b)  shm_List_insert_a_node((a),(b),0,0,a->datasize)

/* Prototypes */
RetValue shm_List_init        ( SHMLIST *listref, int, int );
RetValue shm_List_end         ( SHMLIST **listref );
RetValue shm_List_insert_a_node( SHMLIST *listref, ListPos pos, 
				uchar *userdata, int set, int len );
RetValue shm_List_delete_node ( SHMLIST *listref );
RetValue shm_List_make_first  ( SHMLIST *listref );
RetValue shm_List_get_node    ( SHMLIST *listref, ListPos pos, uchar **userdata );
RetValue shm_List_get_first   ( SHMLIST *listref, uchar **userdata );
RetValue shm_List_get_last    ( SHMLIST *listref, uchar **userdata );
RetValue shm_List_get_next    ( SHMLIST *listref, uchar **userdata );
RetValue shm_List_get_prev    ( SHMLIST *listref, uchar **userdata );
int      shm_List_get_size    ( SHMLIST *listref );
int      shm_List_get_nodesize( SHMLIST *listref );
int      shm_List_determine_memory_req( int, int );
char *   shm_List_get_current_data( SHMLIST *listref );

#endif

/* ---------------------------------------------------------------------- */
/* end */
