/*
 * String handling functions - Header File
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.9 $  $Date: 2003/01/04 12:43:19 $
 * -------------------------------------------------------------------------
 */
#ifndef STRING_LIB_H
#define STRING_LIB_H

#include <phlib/xl_types.h>

/* ---- Structures ---- */
typedef struct _string {
	char *s;
	char *p;
	int  len;
	int  max;
	char f;    /* == 'y' for malloced string, 'n' for conststring */
} CString;

#define CSInit { 0,0,0,0,'n' }

/* ---- MACROS ---- */
#define ToUpper(a) ( ( a > 0x60 && a < 0x7b ) ? (a - 0x20) : a )
#define WrapCString(x,y) \
	{ (x).s=(x).p=(char*)(y); (x).len=strlen(y); (x).max=(x).len; (x).f='n'; }
#define WrapCStringPtr(x,y) \
	{ (x)->s=(x)->p=(char*)(y); (x)->len=strlen(y); (x)->max=(x)->len; (x)->f='n'; }
#define WrapCStringLen(x,y,l) { (x).s=(x).p=(char*)(y); (x).len=l; (x).max=l; (x).f='n'; }
#define WrapCStringMallocedMem(x,y,l) { (x).s=(x).p=(char*)(y); (x).len=(x).max=l; (x).f='y'; }
#define WrapCStringPtrLen(x,y,l) { (x)->s=(x)->p=(char*)(y); (x)->len=l; (x)->max=l; (x)->f='n'; }
#define SetCSMaxSize(x) { (x).len = (x).max; } 
#define SetCSMaxSizePtr(x) { (x)->len = (x)->max; } 
#define MirrorCString(x,y) { (x).s=(y).s; (x).len=(y).len; (x).f=(y).f; (x).p=(y).p; (x).max=(y).max; }
#define MirrorCStringPtr(x,y) { (x)->s=(y)->s; (x)->len=(y)->len; (x)->f=(y)->f; (x)->p=(y)->p; (x)->max=(y)->max; }
#define CSStart(x)    ( (x).s  )
#define CSStartPtr(x) ( (x)->s )
#define CSLen(x)      ( (x).len     )
#define CSLenPtr(x)   ( (x)->len    )
#define CSMax(x)      ( (x).max     )
#define CSMaxPtr(x)   ( (x)->max    )
#define CSChar(x,c)   ( ((x).s[c])  )
#define CSCharPtr(x,c)( ((x)->s[c]) )

/* ---- Prototypes ---- */
void     upper_case     ( CString *str );
void     free_cstring   ( CString *str );
void     skip_lead_space( CString *str );
void     skip_tail_space( CString *str );
void     CStoCharCpy    ( char *chs, CString *cstr, int max_size );
void     reset_cstring  ( CString *str );
int      ends_with_chars( const CString *text, const char* str );
int      begins_with_chars( const CString *text, const char* str );
int      string_left    ( CString *str, int size );
int      string_right   ( CString *str, int size );
int      string_mid     ( CString *str, int st, int fin );
int      get_string_mid ( CString *dst, CString *str, int st, int fin );
int      CScmp          ( CString *s1, CString *s2 );
int      CScmpAlphabetic( CString *s1, CString *s2 );
int      index_cstring  ( CString *, CString *);
int      strcmpCS       ( CString *, const char *);
int      boyermoore_cstring( CString *src, CString *fnd, uint *bmtab );
uint *   boyermoore_init( CString *fnd, uint *bmtab );
RetValue chop_word      ( CString *to, CString *str );
RetValue chop_line      ( CString *to, CString *str, char sep );
RetValue chop_word_last ( CString *to, CString *str );
RetValue chop_line_last ( CString *to, CString *str, char sep );
RetValue CreateCString  ( CString *str, int size );
RetValue extend_cstring ( CString *to,  int size );
RetValue append_cstring ( CString *to, const CString *from );
RetValue append_chars   ( CString *to, const char *from );
RetValue CloneCString   ( CString *str );
RetValue append_intCS   ( CString* dst, int val );
RetValue IntToCS        ( CString* dst, int val );
RetValue CStoInt32      ( CString* src, int32 *val );
RetValue is_cstring_blank( CString* str );

#endif /* STRING_LIB_H */
/* end */
